package DE_EPAGES::Core::t::BaseObject;
use base DE_EPAGES::Core::API::Test;

use strict;
use Test::More tests => 16;
use Test::Exception;
use DE_EPAGES::Core::API::Object::BaseObject;

## no critic qw( ProhibitMagicNumbers ProhibitPrivateSubroutineUse );
sub test {
    my $Object = DE_EPAGES::Core::API::Object::BaseObject->new(
        'Integer' => 12345,
        'Float' => 1234.5678,
        'String' => 'Name',
        'Undef' => undef,
        'Hash' => { Key => 'Value' },
        'SubObject' => DE_EPAGES::Core::API::Object::BaseObject->new(
            'String' => 'SubObject',
        )
    );
    is( $Object->get('Integer'), 12345, 'get Integer' );
    is( $Object->get('Float'), 1234.5678, 'get Float' );
    is( $Object->get('String'), 'Name', 'get String' );
    is( $Object->get('Undef'), undef, 'get Undef' );
    is( $Object->get('Hash')->{'Key'}, 'Value', 'get Hash' );
    is( $Object->get('SubObject')->get('String'), 'SubObject', 'get SubObject' );
    throws_ok { $Object->get('Unknown') } 'DE_EPAGES::Core::API::Error', 'get Unknown';

    # _attribute
    is( $Object->_attribute( FilePathPart => 'dummy' ), 'dummy', '_attribute: set' );
    is( $Object->_attribute('FilePathPart'), 'dummy', '_attribute: get' );
    is( $Object->get('FilePathPart'), 'dummy', '_attribute: get' );

    # tleHash
    my $hVars = $Object->tleHash;
    is( $hVars->{'Integer'}, 12345, 'TLE Integer' );
    is( $hVars->{'Float'}, 1234.5678, 'TLE Float' );
    is( $hVars->{'String'}, 'Name', 'TLE String' );
    is( $hVars->{'Undef'}, undef, 'TLE Undef' );
    is( $hVars->{'Hash'}->{'Key'}, 'Value', 'TLE Hash' );
    is( $hVars->{'SubObject'}->{'String'}, 'SubObject', 'TLE SubObject' );
}

__PACKAGE__->runScript;
