#!perl -w
use strict;
use Test::More tests => 20;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Object::ConfStructureImportHandler;
use DE_EPAGES::Core::API::File qw( GetFileContent DeleteFile);

##no critic qw(ProhibitExcessComplexity ProhibitLongLines ProhibitMagicNumbers)
sub Main {
    RunScript(Sub => sub {
        my $ImportHandler = DE_EPAGES::Core::API::Object::ConfStructureImportHandler->new('epages');
        my $ConfStruct = $ImportHandler->getConfStructure('epages.conf');

        #test 1
        ok(
            (
                $ConfStruct->existsSection('DE_EPAGES::Object') and
                $ConfStruct->existsSection('DE_EPAGES::Mail') and
                $ConfStruct->existsSection('DE_EPAGES::Newsletter') and
                $ConfStruct->existsSection('DE_EPAGES::Store') and
                $ConfStruct->existsSection('DE_EPAGES::Database') and
                $ConfStruct->existsSection('DE_EPAGES::Product')
            ),
            'Sections correct'
        );

        #test 2
        ok(
            (
                $ConfStruct->existsSectionKey('DE_EPAGES::Object', 'SystemDomainName') and
                $ConfStruct->existsSectionKey('DE_EPAGES::Store', 'DefaultWebServiceProxy') and
                $ConfStruct->existsSectionKey('DE_EPAGES::Store', 'DefaultWebServiceLogin') and
                $ConfStruct->existsSectionKey('DE_EPAGES::Store', 'DefaultWebServicePassword') and
                $ConfStruct->existsSectionKey('DE_EPAGES::Store', 'WebServiceVerifyHostname')
            ),
            'Section keys correct'
        );

        #test section keys
        ok(
            ($ConfStruct->getSectionKeyValue('DE_EPAGES::Object', 'SystemDomainName') eq 'testdomain.intern.epages.de'),
            'Section keys correct 1st'
        );
        ok(
            ($ConfStruct->getSectionKeyValue('DE_EPAGES::Store', 'DefaultWebServiceProxy') eq 'http://_SystemDomainName_/epages/_Store_.soap'),
            'Section keys correct 2nd'
        );
        ok(
            ($ConfStruct->getSectionKeyValue('DE_EPAGES::Store', 'DefaultWebServiceLogin') eq '/Users/wsadmin'),
            'Section keys correct 3rd'
        );
        ok(
            ($ConfStruct->getSectionKeyValue('DE_EPAGES::Store', 'DefaultWebServicePassword') eq 'wsadmin'),
            'Section keys correct 4th'
        );
        ok(
            ($ConfStruct->getSectionKeyValue('DE_EPAGES::Store', 'WebServiceVerifyHostname') eq '1'),
            'Section keys correct 5th'
        );

        my $MaxPosition = $ConfStruct->getSectionKeyCommentsMaxPosition('DE_EPAGES::Store');

        ok($MaxPosition eq '20', 'correct max section key position');

        $MaxPosition = $ConfStruct->getSectionKeyCommentsMaxPosition('DE_EPAGES::Product');

        ok($MaxPosition eq '3', 'correct max section key position 2nd');

        $MaxPosition = $ConfStruct->getSectionsMaxPosition();

        ok($MaxPosition eq '6', 'correct max sections position');

        my $Comments = [
            '# Values: String',
            '# This parameter is used during the installation of the Object Cartridge and will then stored in the Sytem.DomainName Attribute to uses this further way',
            '',
            '#IsFilePathPartUsed=0',
            '# Values:(0..1)',
            '# Values:(0..1)',
            '# influences the bahaviour when products are deleted or duplicated.',
            '# If IsFilePathPartUsed is false, then images are duplicated',
            '#  when a product is duplicated and images are deleted when a product is deleted.',
            '# Otherwise when a product is duplicated, the duplicate will share the images',
            '#  of the original product and when a product is deleted, the images are not deleted.',
            '# In this case, merchants must delete unused images by other means,',
            '# for example using FTP. (see also: ePages Knowledge base article 1272)',
            '',
            '#IsWebServerScriptNamePartUnique=0',
            '# Values:(0..1)',
            '# influences the SSL URL that is generated for shops',
            '#  that do not have their own domain name and SSL certificate.',
            '# If IsWebServerScriptNamePartUnique is true, the SSL URL is',
            '#  formed from System.DomainName and Shop.WebServerScriptNamePart.',
            '#  This allows to use relative links to objects in the same shop,',
            '#  such as ObjectPath=Pages/Imprint. For this to work reliable,',
            '#  the short URL of all shops must be unique in the system.',
            '# IsWebServerScriptNamePartUnique should not be changed before the first shop is created.',
            '# Use the refreshPoolDB.pl script (Object cartridge)',
            '# to create the required ASPoolDB entries for existing shops.',
            '',
            '#ExpandWWWDomain=0',
            '# Values:(0..1)',
            '# allows the use of domain names with and without "www." at the beginning.',
            '# Note that both names must have a DNS entry with the same IP address.',
            '# The domain name entered in PBO (primary name) will always be used in the shop.',
            '# The alternate name (plus or minus the \'www.\') can be used only to',
            '# enter the shop for the first time,',
            '# thereafter, the domain name will be replaced by the primary name.',
            '# The provider is responible for assigning unique top level domains to individual shops.',
            '# It may still be possible to assign "www.myshop.com" to one shop and "myshop.com"',
            '# to another shop, but the alternate name won\'t work as expected in this case.',
            '# When using the default setting (ExpandWWWDomain=0 or not set),',
            '# the domains "www.myshop.com" and "myshop.com" are completely independent and',
            '# may therefore be assigned to different shops.',
            '# Enabeling ExpandWWWDomain will affect only new shops.',
            '# Use the refreshPoolDB.pl script (Object cartridge)',
            '# to expand the domain names of existing shops.',
            ''
        ];

        is_deeply($Comments, $ConfStruct->getSectionComments('DE_EPAGES::Object'),'comments correct 1st');


        $ConfStruct->exportConf('Test.conf');
        my $rContent1 = GetFileContent('Test.conf');
        my $rContent2 = GetFileContent('epages.conf');

        ok($$rContent1 eq $$rContent2, 'correct export');

        $ConfStruct->removeSection('DE_EPAGES::Object');

        ok(not($ConfStruct->existsSection('DE_EPAGES::Object')),'remove section');

        $ConfStruct->exportConf('Test.conf');
        $rContent1 = GetFileContent('Test.conf');
        $rContent2 = GetFileContent('epages_cut1.conf');

        ok($$rContent1 eq $$rContent2, 'remove section and comments');

        $ConfStruct->removeSectionKey('DE_EPAGES::Store', 'DefaultWebServiceLogin');

        ok(not ($ConfStruct->existsSectionKey('DE_EPAGES::Store', 'DefaultWebServiceLogin')),'remove section key 1st');

        $ConfStruct->exportConf('Test.conf');
        $rContent1 = GetFileContent('Test.conf');
        $rContent2 = GetFileContent('epages_cut2.conf');

        ok($$rContent1 eq $$rContent2, 'remove section key 2nd');

        my $ImpConfStruct = $ImportHandler->getConfStructure('epages_imp.conf');

        $ConfStruct->importConfStructure($ImpConfStruct);

        ok($ConfStruct->existsSection('DE_EPAGES::Object'),'import structure 1st');
        ok($ConfStruct->existsSectionKey('DE_EPAGES::Store', 'DefaultWebServiceLogin'),'import structure 2nd');
        is_deeply($Comments, $ConfStruct->getSectionComments('DE_EPAGES::Object'),'import structure 3rd');

        $ConfStruct->exportConf('Test.conf');
        $rContent1 = GetFileContent('Test.conf');
        $rContent2 = GetFileContent('epages.conf');

        ok($$rContent1 eq $$rContent2, 'import structure 4th');

        DeleteFile('Test.conf');

    }, Debug=>1);
}

Main();

1;