use strict;
use lib ($ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES/Core/Scripts');

use DE_EPAGES::Core::API::Script qw( RunScript );
use Perl::Critic;
use Test::More tests => 10;

sub Main {

    my $critic = Perl::Critic->new(
        -profile  => 'NONE',
        -severity => 5
    );

    ok( defined $critic, 'Perl::Critic initialized' );
    $critic->add_policy( -policy => 'Perl::Critic::Policy::DE_EPAGES::Modules::RequireUtf8BOM' );
    my @policies = $critic->policies();
    my $policyLoaded = scalar grep { $_ =~ 'DE_EPAGES::Modules::RequireUtf8BOM' }
    @policies;
    ok( $policyLoaded, 'Policy loaded' );

    my @violations;
    @violations = $critic->critique( 'Perl-Critic/NoUseUtf8.txt' );
    is(scalar @violations, 1, 'no "use utf-8"') or diag(@violations);
    my $text = 'does not contain "use utf'.'8;"';
    like $violations[0], qr{\Q$text\E}, 'no "use utf-8"';

    @violations = $critic->critique( 'Perl-Critic/Utf8NoBOM.txt' );
    is(scalar @violations, 1, 'no utf-8 BOM') or diag(@violations);
    like $violations[0], qr{has no utf-8 BOM}, 'no utf-8 BOM';

    @violations = $critic->critique( 'Perl-Critic/IsoParagraph.txt' );
    is(scalar @violations, 1, 'ISO paragraph in file with BOM') or diag(@violations);
    like $violations[0], qr{appears to contain iso-8859-1}, 'ISO paragraph in file with BOM';

    @violations = $critic->critique( 'Perl-Critic/Utf7BOM.txt' );
    is(scalar @violations, 1, 'utf-7 BOM') or diag(@violations);
    like $violations[0], qr{Unsupported character encoding}, 'utf-7 BOM';
}

RunScript( Sub => \&Main );
