/*global define*/
define('de_epages/externalcontent/gadgetdialog/config-view', [
    'jquery',
    'backbone',
    './constants',

    'ep/ui/tooltip',
    'ep/ui/input'
], function ($, Backbone, CONSTANTS) {

    'use strict';

    var configView = Backbone.View.extend({

        // alias: 'gadget name', e.g. 'rss'

        // template: template_function, i.e. return value of require(['$tmpl!./rss-view'])

        // dict: dictionary

        tagName: 'div',

        setOption: function (key, value, refreshPreview) {
            var self = this;
            self.gOptions[key] = value;
            if (refreshPreview) {
                self.renderPreview();
            }
            return self;
        },

        setOptions: function (options) {
            var self = this,
                key;
            for (key in options) {
                if (options.hasOwnProperty(key)) {
                    self.setOption(key, options[key]);
                }
            }
            return self;
        },

        renderPreview: function () {
            var self = this,
                preview = self.elements.preview,
                parent = preview.parent(),
                gadgetPath = CONSTANTS.gWidgetNamespace + self.alias.charAt(0).toUpperCase() + self.alias.slice(1);
            preview.remove();
            preview = self.elements.preview = $('<div class="PreviewNode">').appendTo(parent);
            preview[gadgetPath](self.gOptions);
        },

        onSelect: function (e, noPreview) {
            var target = e.target,
                selected = $(target).find(':selected');
            this.setOption(target.name, selected.val(), noPreview ? false : true);
        },

        // Update options on input.
        onInput: function (e, noPreview) {
            var target = e.target,
                $target = $(target),
                siblings,
                valueArray = [];

            if (target.type === 'checkbox') {
                siblings = $target.closest('fieldset');
                if (siblings.length === 0) {
                    siblings = $target.closest('form');
                }
                siblings = siblings.find(':checkbox[name="' + target.name + '"]');
                if (siblings.length > 1) {
                    siblings.filter(':checked').each(function () {
                        valueArray.push($(this).val());
                    });
                    this.setOption(target.name, valueArray, noPreview ? false : true);
                } else {
                    this.setOption(target.name, target.checked, noPreview ? false : true);
                }
            } else { //type text
                this.setOption(target.name, target.value, noPreview ? false : true);
            }
        },

        onSubmit: function () {
            return false;
        },

        onRadio: function (e) {
            var target = e.target,
                self = this;
            if (target.value !== self.gOptions[target.name]) {
                // Set *self.gOptions* according to newly chosen type (i.e. *target.value*) and
                // show fieldset with name *target.value*, hide the others.
                self.gOptions = {};
                self.gOptions[target.name] = target.value;
                self.$el.find('fieldset').not('[name="' + target.name + '"], [name="' + target.value + '"]').attr('disabled', true).addClass('HideElement');
                self.$el.find('fieldset[name="' + target.value + '"]').attr('disabled', false).removeClass('HideElement');
                // Set remaining *self.gOptions* according to fieldset shown.
                self.$el.find('fieldset[name="' + target.value + '"] :input').trigger('change', true);
                self.renderPreview();
            }
        },

        onApply: function () {
            // VALIDATION
            // https://jira.epages.de/browse/EPG-26271
            if (this.$('form').find('input:invalid').length) {
                if (!this.fakesubmit) {
                    var $form = this.$('form');
                    this.fakesubmit = $('<button type="submit">').addClass('HideElement').attr('form', $form.attr('id')).appendTo($form);
                }
                this.fakesubmit.trigger('click');
                return false;
            }
            if (this.elements.verifyingUrl && (this.elements.verifyingUrl.hasClass('wait-icon') || this.elements.verifyingUrl.hasClass('fail-icon'))) {
                return false;
            }
            return true;
        },

        getAPI: function () {
            // Overwritten by the individual gadget views
        },

        // Init options of the gadget.
        initialize: function () {
            var self = this,
                gotNoOptions = $.isEmptyObject(self.options);
            self.differTypes = $.isArray(CONSTANTS.gDefaults[self.alias]);
            self.gDefaults = $.extend(true, self.differTypes ? [] : {}, CONSTANTS.gDefaults[self.alias]);
            self.plugin = CONSTANTS.gNamespace + '.ui.' + self.alias;
            self.getAPI();
            if (self.differTypes) {
                if (gotNoOptions) {
                    self.gOptions = self.gDefaults[0];
                } else {
                    self.gOptions = self.options;
                }
            } else {
                self.gOptions = gotNoOptions ? self.gDefaults : self.options;
            }
            self.verifyUrl = null;
        },
	
		// overwriteable method
		// give views which inherit from this view the possiblity to extend template options
        addAdditionalTemplateOptions: function(gOptions){},

        render: function () {
            var self = this,
                types = [],
                defaults,
                renderedTemplate,
                tmplObject,
                i;

            if (self.differTypes) {
                // Merge *self.gOptions* in *types* for templating. (See below.)
                for (i = 0; i < self.gDefaults.length; i = i + 1) {
                    defaults = self.gDefaults[i];
                    types.push(defaults.type === self.gOptions.type ? self.gOptions : defaults);
                }
            }

            tmplObject = self.differTypes ? {
                    'types': types,
                    'gOptions': self.gOptions
                } : self.gOptions;

			// give views which inherit from this view the possiblity to extend template options
            self.addAdditionalTemplateOptions(tmplObject);

            renderedTemplate = self.template(tmplObject).dictParse(self.dict, true);

            renderedTemplate.find('.Tooltip').each(function () {
                $(this).uiTooltip({
                    interactive: true
                });
            });
            self.$el.append(renderedTemplate);
            renderedTemplate.find(':input').each(function () {
                if (this.placeholder) {
                    this.placeholder = self.dict.translate(this.placeholder);
                }
                $(this).uiInput();
            });
            self.elements = renderedTemplate.tmplItem('elements'); // preview element z.B.
            self.renderPreview();
            return this;
        }
    }, {
        events: {
            'submit form': 'onSubmit',
            'change input[type!="radio"]': 'onInput',
            'change input[type="radio"]': 'onRadio',
            'change select': 'onSelect'
        }
    });

    return configView;

});