use strict;
use Test::More tests => 2;
use DE_EPAGES::ExternalContent::API::XML::GetGeoAddressHandler qw ( ParseAddress );
use DE_EPAGES::ExternalContent::API::XML::GetRouteHandler qw ( ParseRoute );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );

my $XMLAddress = <<XML;
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE IGeocoding SYSTEM "http://xml.klickurl.de/mrf/IGeocoding/IGeocoding.dtd">
<IGeocoding version="5.0">
<status errorcode="0" errortext="OK"></status>
<body>
    <list number="1">
        <address QoS="50">
            <zip WLD="0">4</zip>
            <city WLD="8">Arendal</city>
            <district WLD="0">Asdal</district>
            <country WLD="0">N</country>
            <longitude>51589620</longitude>
            <latitude>63273997</latitude>
        </address>
    </list>
</body>
</IGeocoding>
XML

my $XMLRoute = <<XML;
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE IRouting SYSTEM "http://xml.klickurl.de/mrf/IRouting/IRouting.dtd">
  <IRouting version="5.0">
    <status errorcode="0" errortext="OK">
  </status>
    <body>
      <getroute id="821748180">
        <description number="23">
          <rectangle location="route">
            <point_intern x="51580016" y="57522902" />
            <point_intern x="55695215" y="54539247" />
          </rectangle>
          <rectangle location="start">
            <point_intern x="52162901" y="55149618" />
            <point_intern x="52172901" y="55139618" />
          </rectangle>
          <rectangle location="end">
            <point_intern x="55102329" y="56940017" />
            <point_intern x="55112329" y="56930017" />
          </rectangle>
          <statistics>
            <routelength value="437.34"/>
            <motorway value="401.7084"/>
            <routeduration value="4 h 23 min"/>
            <fuelconsumption value="39.36"/>
          </statistics>
          <point>
            <number>1</number>
            <time>00:00</time>
            <distance_total>0.00</distance_total>
            <recommendation>Abfahrt Waisengasse</recommendation>
            <street>Waisengasse</street>
            <streettype>Sonstige Str.</streettype>
            <streetinfo>Bad Wildungen</streetinfo>
            <distance>0.00</distance>
            <direction>N</direction>
            <rectangle>
              <point_intern x="52163905" y="55148610" />
              <point_intern x="52171905" y="55140610" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>2</number>
            <time>00:00</time>
            <distance_total>0.00</distance_total>
            <recommendation>Nach rechts auf Brunnenstr.</recommendation>
            <street>Brunnenstr.</street>
            <streettype>Sonstige Str.</streettype>
            <streetinfo>Bad Wildungen</streetinfo>
            <distance>0.20</distance>
            <direction>N</direction>
            <rectangle>
              <point_intern x="52163901" y="55148619" />
              <point_intern x="52171901" y="55140619" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>3</number>
            <time>00:00</time>
            <distance_total>0.20</distance_total>
            <recommendation>Halb rechts auf Kaiserlindenplatz</recommendation>
            <street>Kaiserlindenplatz</street>
            <streettype>Verbindungsstr.</streettype>
            <streetinfo>Bad Wildungen</streetinfo>
            <distance>0.11</distance>
            <direction>O</direction>
            <rectangle>
              <point_intern x="52165790" y="55149247" />
              <point_intern x="52173790" y="55141247" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>4</number>
            <time>00:00</time>
            <distance_total>0.31</distance_total>
            <recommendation>Nach links auf Waldschmidtstr.</recommendation>
            <street>Waldschmidtstr.</street>
            <streettype>Verbindungsstr.</streettype>
            <streetinfo>Bad Wildungen</streetinfo>
            <distance>0.13</distance>
            <direction>SO</direction>
            <rectangle>
              <point_intern x="52166219" y="55148408" />
              <point_intern x="52174219" y="55140408" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>5</number>
            <time>00:01</time>
            <distance_total>0.44</distance_total>
            <recommendation>Nach rechts auf Bahnhofstr.</recommendation>
            <street>Bahnhofstr.</street>
            <streettype>Hauptverkehrsstr.</streettype>
            <streetinfo>Bad Wildungen</streetinfo>
            <distance>0.40</distance>
            <direction>NO</direction>
            <rectangle>
              <point_intern x="52166778" y="55149418" />
              <point_intern x="52174778" y="55141418" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>6</number>
            <time>00:01</time>
            <distance_total>0.84</distance_total>
            <recommendation>Nach links auf Berliner Str.</recommendation>
            <street>Berliner Str.</street>
            <streettype>Hauptverkehrsstr.</streettype>
            <streetinfo>Bad Wildungen</streetinfo>
            <distance>1.97</distance>
            <direction>O</direction>
            <rectangle>
              <point_intern x="52170576" y="55150410" />
              <point_intern x="52178576" y="55142410" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>7</number>
            <time>00:03</time>
            <distance_total>2.81</distance_total>
            <recommendation>Nach rechts auf B485</recommendation>
            <street>B485</street>
            <streettype>Hauptverkehrsstr.</streettype>
            <streetinfo>Bad Wildungen</streetinfo>
            <distance>0.45</distance>
            <direction>NO</direction>
            <rectangle>
              <point_intern x="52183808" y="55159775" />
              <point_intern x="52191808" y="55151775" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>8</number>
            <time>00:04</time>
            <distance_total>3.26</distance_total>
            <recommendation>Halb rechts auf B485</recommendation>
            <street>B485 Richtung FritzlarKasselMarburg</street>
            <streettype>Bundesstr.</streettype>
            <streetinfo></streetinfo>
            <distance>10.76</distance>
            <direction>SO</direction>
            <rectangle>
              <point_intern x="52186603" y="55156518" />
              <point_intern x="52194603" y="55148518" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>9</number>
            <time>00:13</time>
            <distance_total>14.01</distance_total>
            <recommendation>Nach rechts auf A49</recommendation>
            <street>A49 Richtung Kassel</street>
            <streettype>Bundesstr.</streettype>
            <streetinfo>AS Wabern (15)</streetinfo>
            <distance>31.14</distance>
            <direction>O</direction>
            <rectangle>
              <point_intern x="52281556" y="55132083" />
              <point_intern x="52289556" y="55124083" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>10</number>
            <time>00:30</time>
            <distance_total>45.16</distance_total>
            <recommendation>Verlassen Sie bei AK Kreuz Kassel-Mitte (2)(79) die Autobahn und fahren Sie auf die A7</recommendation>
            <street>A7 Richtung HamburgHannover</street>
            <streettype>Autobahn</streettype>
            <streetinfo>AK Kreuz Kassel-Mitte (2)(79)</streetinfo>
            <distance>114.02</distance>
            <direction>NO</direction>
            <rectangle>
              <point_intern x="52436796" y="55331985" />
              <point_intern x="52444796" y="55323985" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>11</number>
            <time>01:32</time>
            <distance_total>159.17</distance_total>
            <recommendation>Verlassen Sie bei AD Dreieck Salzgitter (23)(64) die Autobahn und fahren Sie auf die A39</recommendation>
            <street>A39 Richtung BerlinBraunschweigSalzgitter</street>
            <streettype>Autobahn</streettype>
            <streetinfo>AD Dreieck Salzgitter (23)(64)</streetinfo>
            <distance>49.75</distance>
            <direction>NO</direction>
            <rectangle>
              <point_intern x="52841208" y="56248862" />
              <point_intern x="52849208" y="56240862" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>12</number>
            <time>01:59</time>
            <distance_total>208.92</distance_total>
            <recommendation>Verlassen Sie bei AK Kreuz Wolfsburg/Königslutter (7)(58) die Straße</recommendation>
            <street></street>
            <streettype>Autobahn</streettype>
            <streetinfo>AK Kreuz Wolfsburg/Königslutter (7)(58)</streetinfo>
            <distance>0.23</distance>
            <direction>NO</direction>
            <rectangle>
              <point_intern x="53193320" y="56512219" />
              <point_intern x="53201320" y="56504219" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>13</number>
            <time>01:59</time>
            <distance_total>209.14</distance_total>
            <recommendation>Verlassen Sie bei AK Kreuz Wolfsburg/Königslutter (7)(58) die Autobahn und fahren Sie auf die A2</recommendation>
            <street>A2</street>
            <streettype>Autobahn</streettype>
            <streetinfo>AK Kreuz Wolfsburg/Königslutter (7)(58)</streetinfo>
            <distance>161.87</distance>
            <direction>N</direction>
            <rectangle>
              <point_intern x="53194060" y="56514340" />
              <point_intern x="53202060" y="56506340" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>14</number>
            <time>03:24</time>
            <distance_total>371.01</distance_total>
            <recommendation>Fahren Sie links weiter auf A10</recommendation>
            <street>A10 Richtung Berlin-ZentrumFrankfurt (Oder)Potsdam</street>
            <streettype>Autobahn</streettype>
            <streetinfo>AD Dreieck Potsdam (19)(1)</streetinfo>
            <distance>11.22</distance>
            <direction>O</direction>
            <rectangle>
              <point_intern x="54670513" y="56611557" />
              <point_intern x="54678513" y="56603557" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>15</number>
            <time>03:31</time>
            <distance_total>382.23</distance_total>
            <recommendation>Verlassen Sie bei AD Dreieck Nuthetal (9)(16) die Autobahn und fahren Sie auf die A115</recommendation>
            <street>A115 Richtung Berlin-ZehlendorfBerlin-ZentrumPotsdam-Zentrum</street>
            <streettype>Autobahn</streettype>
            <streetinfo>AD Dreieck Nuthetal (9)(16)</streetinfo>
            <distance>27.55</distance>
            <direction>O</direction>
            <rectangle>
              <point_intern x="54776187" y="56629535" />
              <point_intern x="54784187" y="56621535" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>16</number>
            <time>03:46</time>
            <distance_total>409.78</distance_total>
            <recommendation>Fahren Sie links weiter auf A115</recommendation>
            <street>A115 Richtung KurfürstendammSchönefeldWilmersdorf</street>
            <streettype>Autobahn</streettype>
            <streetinfo></streetinfo>
            <distance>6.18</distance>
            <direction>NO</direction>
            <rectangle>
              <point_intern x="54889442" y="56855094" />
              <point_intern x="54897442" y="56847094" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>17</number>
            <time>03:50</time>
            <distance_total>415.96</distance_total>
            <recommendation>Verlassen Sie bei AS Wexstraße (16) die Autobahn</recommendation>
            <street> Richtung Innsbrucker Platz</street>
            <streettype>Autobahn</streettype>
            <streetinfo>AS Wexstraße (16)</streetinfo>
            <distance>0.74</distance>
            <direction>O</direction>
            <rectangle>
              <point_intern x="54935916" y="56841925" />
              <point_intern x="54943916" y="56833925" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>18</number>
            <time>03:51</time>
            <distance_total>416.70</distance_total>
            <recommendation>Nach links auf Hauptstr.</recommendation>
            <street>Hauptstr.</street>
            <streettype>Hauptverkehrsstr.</streettype>
            <streetinfo>AS Innsbrucker Platz (17)</streetinfo>
            <distance>3.54</distance>
            <direction>O</direction>
            <rectangle>
              <point_intern x="54943105" y="56843004" />
              <point_intern x="54951105" y="56835004" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>19</number>
            <time>03:57</time>
            <distance_total>420.24</distance_total>
            <recommendation>Fahren Sie links weiter auf Potsdamer Str. (B1)</recommendation>
            <street>Potsdamer Str. (B1)</street>
            <streettype>Bundesstr.</streettype>
            <streetinfo>Berlin Tiergarten</streetinfo>
            <distance>3.64</distance>
            <direction>N</direction>
            <rectangle>
              <point_intern x="54956477" y="56874534" />
              <point_intern x="54964477" y="56866534" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>20</number>
            <time>04:03</time>
            <distance_total>423.88</distance_total>
            <recommendation>Fahren Sie links weiter auf Grunerstr. (B2)</recommendation>
            <street>Grunerstr. (B2)</street>
            <streettype>Bundesstr.</streettype>
            <streetinfo>Berlin Mitte</streetinfo>
            <distance>0.86</distance>
            <direction>NO</direction>
            <rectangle>
              <point_intern x="54984441" y="56892213" />
              <point_intern x="54992441" y="56884213" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>21</number>
            <time>04:04</time>
            <distance_total>424.74</distance_total>
            <recommendation>Nach rechts auf Mollstr.</recommendation>
            <street>Mollstr.</street>
            <streettype>Bundesstr.</streettype>
            <streetinfo>Berlin Mitte</streetinfo>
            <distance>12.30</distance>
            <direction>NO</direction>
            <rectangle>
              <point_intern x="54989123" y="56899311" />
              <point_intern x="54997123" y="56891311" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>22</number>
            <time>04:23</time>
            <distance_total>437.03</distance_total>
            <recommendation>Nach links auf Hellersdorfer Weg</recommendation>
            <street>Hellersdorfer Weg</street>
            <streettype>Hauptverkehrsstr.</streettype>
            <streetinfo>Ahrensfelde Eiche-Süd</streetinfo>
            <distance>0.25</distance>
            <direction>O</direction>
            <rectangle>
              <point_intern x="55102663" y="56936551" />
              <point_intern x="55110663" y="56928551" />
            </rectangle>
            <video>-1</video>
          </point>
          <point>
            <number>23</number>
            <time>04:24</time>
            <distance_total>437.28</distance_total>
            <recommendation>Ankunft Hellersdorfer Weg</recommendation>
            <street>Hellersdorfer Weg</street>
            <streettype>Sonstige Str.</streettype>
            <streetinfo>Ahrensfelde Eiche-Süd</streetinfo>
            <distance>0.00</distance>
            <direction>N</direction>
            <rectangle>
              <point_intern x="55103330" y="56939016" />
              <point_intern x="55111330" y="56931016" />
            </rectangle>
            <video>-1</video>
          </point>
        </description>
      </getroute>
    </body>
  </IRouting>
XML

#========================================================================================
#   MAIN
#========================================================================================
RunScript(
    Sub => sub {
        my $hResult;
        eval {
            $hResult = ParseAddress(\$XMLAddress);
        };
        ok(defined $hResult && ! ExistsError(), 'ParseAddress') or diag(GetError->longMessage());
        $hResult = undef;
        eval {
            $hResult = ParseRoute(\$XMLRoute);
        };
        ok(defined $hResult && ! ExistsError(), 'ParseRoute') or diag(GetError->longMessage());

    }
);