package DE_EPAGES::GoogleAnalytics::t::checkRedirection;
use base DE_EPAGES::Database::API::Test;

use strict;
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::WebInterface::API::FakeRequest;
use Test::More;
use HTTP::Status qw ( HTTP_MOVED_PERMANENTLY );

my $ROOTPATH = '/Shops';
local $ENV{'EPAGES_TESTSITE'} = 'Store' unless defined $ENV{'EPAGES_TESTSITE'};

sub test {

    local $DE_EPAGES::Object::API::Language::LANGUAGEID = 1;
    my $FakeRequester = DE_EPAGES::WebInterface::API::FakeRequest->new;

    my $Shop = LoadObjectByPath('/Shops/TestShop/');
    ok($Shop->alias eq 'TestShop', 'Shop loaded');
    $Shop->set({'DomainName' => 'example.com'});
    is($Shop->get('DomainName'), 'example.com', 'domain correct');
    my $Cat     = $Shop->get('Categories')->child('MinimalCategory');
    my $Product = $Shop->child('Products')->child('MinimalProduct1');

    my $BaseURL = 'http://example.com/Shops/TestShop/';
    my $MobileBaseURL = 'http://example.com/Mobile/Shops/TestShop/';
    my $ShopWebURL = $Shop->get('WebUrl') . '/';

    my $hURLsToTest = {
        # Startpage, Homesite.
        $BaseURL . 'HomeSite'  => $ShopWebURL . '?ViewAction=View',
        $BaseURL . 'StartPage' => $Shop->get('Categories')->get('WebUrl'),
        # Search targets.
        $BaseURL . 'SearchMask'    => $ShopWebURL . '?ViewAction=ViewDetailSearchProducts',
        $BaseURL . 'Search'        => $ShopWebURL . '?ViewAction=DetailSearchProducts',
        $BaseURL . 'FacetedSearch' => $ShopWebURL . '?ViewAction=FacetedSearchProducts',
        # Additional tracking target for class shop.
        $BaseURL . 'Newsletter'        => $ShopWebURL . '?ViewAction=ViewNewsletter',
        $BaseURL . 'Registration'      => $ShopWebURL . '?ViewAction=ViewRegistration',
        $BaseURL . 'ProductComparison' => $ShopWebURL . '?ViewAction=ViewProductComparison',
        # ContentObjects e.g. product, category.
        $BaseURL . 'Products/MinimalProduct1'         => $Product->get('WebUrl'),
        $MobileBaseURL . 'Products/MinimalProduct1'   => $Product->get('WebUrlMobile'),
        $BaseURL . 'Categories/MinimalCategory'       => $Cat->get('WebUrl'),
        $MobileBaseURL . 'Categories/MinimalCategory' => $Cat->get('WebUrlMobile'),
        # Target which redirect to info page.
        # Targets on User class.
        $BaseURL . 'MyAccount'     => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
        $BaseURL . 'PersonalData'  => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
        $BaseURL . 'OrderTracking' => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
        $BaseURL . 'ShoppingLists' => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
        $BaseURL . 'Addresses'     => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
        $BaseURL . 'PassWD'        => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
        # Targets on ShoppingList class.
        $BaseURL . 'ShoppingList' => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
        # Targets on CustomerOrder class.
        $BaseURL . 'Basket/BasketAddress/Offer/Order' => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
        $BaseURL . 'OldOrder'                         => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
        # Targets on Basket class.
        $BaseURL . 'Basket'                     => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
        $BaseURL . 'BasketShippingOptions'      => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
        $BaseURL . 'Basket/BasketAddress'       => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
        $BaseURL . 'Basket/BasketAddress/Offer' => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
        $BaseURL . 'Basket/Payment'             => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
        $BaseURL . 'Basket/HirePurchase'        => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
        $BaseURL . 'Basket/Confirmation'        => $ShopWebURL . '?ViewAction=ViewTrackedPageInfo',
    };

    foreach my $SiteURL (keys %{$hURLsToTest}) {
        my $Response = $FakeRequester->get( $SiteURL );
        ok($Response->is_redirect, "$SiteURL: request was redirected");
        is($Response->code, HTTP_MOVED_PERMANENTLY, "$SiteURL: status code: Moved Permanently.");
        ok($Response->header('Location') eq $hURLsToTest->{$SiteURL}, "$SiteURL: correct redirected.");
    }

    done_testing();
}

sub init {
    return if $ENV{'SKIP_CLEANUP'};
    ImportObjects('import.xml', $ROOTPATH);
}

sub cleanup {
    return if $ENV{'SKIP_CLEANUP'};
    my $Shops = LoadObjectByPath($ROOTPATH);
    $Shops->child('TestShop')->delete if $Shops->existsChild('TestShop');
}

__PACKAGE__->runScript;
