#!perl -w
use Test::More tests => 6;
use strict;
use File::Basename qw(dirname);
use File::Temp qw(tempdir);
use LWP::UserAgent;
use LWP::Simple;

use DE_EPAGES::Core::API::File qw(CreateDirectory WriteFile DeleteDirectory);
use DE_EPAGES::Core::API::Script qw(RunScript);
use DE_EPAGES::Installer::API::NodeJS qw(GetNodeJSVersion GetNodeJSZipUrl
                                         UpdateNodeJS UnpackNodeJS);

sub Main {
    my $EpagesVersion = '6.12.8';
    my $ExpectedVersion = "$EpagesVersion-22";

    my $Content = <<"HERE"
$ExpectedVersion
HERE
;
    my $TmpDir = tempdir();
    my $NodeDir = "$TmpDir/bin/nodejs.d";

    CreateDirectory($NodeDir);
    WriteFile("$NodeDir/epages.version", \$Content);

    my $Version = GetNodeJSVersion($NodeDir);
    ok( $ExpectedVersion eq $Version, "GetNodeJSVersion: $ExpectedVersion eq $Version" );

    my $NodeJSZip = "epages-nodejs-$EpagesVersion.zip";
    my $ExpectedUrl = "http://ep-server/yum/windows/3rd-party/32/$NodeJSZip";
    my $DownloadUrl = GetNodeJSZipUrl($EpagesVersion);
    ok( $ExpectedUrl eq $DownloadUrl, "GetNodeJSZipUrl: $ExpectedUrl eq $DownloadUrl" );

    my $UserAgent = LWP::UserAgent->new();
    my $Response = $UserAgent->head(dirname($ExpectedUrl));

  SKIP: {
      skip($Response->message. ': '. dirname($ExpectedUrl), 4) unless $Response->is_success;
    }

    $EpagesVersion = '6.17.0';
    $NodeJSZip = "epages-nodejs-$EpagesVersion.zip";
    $ExpectedUrl = "http://ep-server/yum/$EpagesVersion/windows/3rd-party/32/$NodeJSZip";

    local $ENV{EPAGES_PERL} = $TmpDir;
    my $Result = UpdateNodeJS($EpagesVersion, $ExpectedUrl);
    ok( $Result == 1, "UpdateNodeJS: updated == $Result" );
    $Result = UpdateNodeJS($EpagesVersion, $ExpectedUrl);
    ok( $Result == 0, "UpdateNodeJS: updated == $Result" );

    $Version = GetNodeJSVersion($NodeDir);
    ok( $Version =~ /^$EpagesVersion-/, "GetNodeJSVersion: $Version =~ /^$EpagesVersion-/" );

    my $UnpackDir = UnpackNodeJS("$TmpDir/bin/$NodeJSZip", undef, {dont_move => 1});
    ok( $UnpackDir eq "$TmpDir/bin/nodejs-new", "UnpackNodeJS: $UnpackDir eq $TmpDir/bin/nodejs-new" );

    DeleteDirectory($TmpDir, {recursive => 1});
}

RunScript( Sub => \&Main );
1;
