/*
 * @copyright		© Copyright 2006-2013, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages/shipping/inc/deliverysettingsdialog
 *
 * @revision		$Revision: 1.9 $
 */

define("de_epages/shipping/inc/deliverysettingsdialog", [
	"jquery",
	"ep",

	"jquery/ui/draggable",
	"ep/ajax",
	"ep/alert",
	"ep/ui/dialog"
], function ($) {

	var deliverysettingsdialog;

	return (deliverysettingsdialog = {
		ShippingFulfillmentSettingsDialog: null,

		ShippingFulfillmentID: null,

		ShippingFulfillmentSettingsDialog_1stOpen: false,

		lock: function (lock) {
			if (this.ShippingFulfillmentSettingsDialog != null) {
				this.ShippingFulfillmentSettingsDialog.parent()
					.busy(
				(lock) ? 'show' : 'hide');
			}
		},

		open: function (objectId) {
			this.ShippingFulfillmentID = objectId;
			this.ShippingFulfillmentSettingsDialog_1stOpen = true;
			$('.Dialog')
				.busy('show');
			ep.ajax({
				url: "?ViewAction=MBO-ShippingFulfillmentSettingsDialog&ObjectID=" + objectId
			})
			.done(function (data) {
				deliverysettingsdialog.show(data);
			})
			.fail(function () {
				deliverysettingsdialog.ajaxError();
			})
			.always(function () {
				$('.Dialog')
					.busy('hide');
			});
		},

		show: function (data) {
			if (this.ShippingFulfillmentSettingsDialog == null) {
				this.ShippingFulfillmentSettingsDialog = $('#ShippingSettingsDialog')
					.html(data)
					.uiDialog({
					form: {
						class: 'ep-js'
					},
					title: $('#ShippingSettingsDialogLabels > #DialogTitle')
						.html(),
					modal: true,
					autoOpen: false,
					draggable: false,
					resizable: false,
					width: 950,
					height: 600,
					buttons: [{
						text: $('#ShippingSettingsDialogLabels > #OkButton')
							.html(),
						class: 'SaveButton',
						click: function () {
							deliverysettingsdialog.submit();
						}
					}, {
						text: $('#ShippingSettingsDialogLabels > #CloseButton')
							.html(),
						click: function () {
							deliverysettingsdialog.close();
						}
					}]
				});
			}
			else {
				this.ShippingFulfillmentSettingsDialog.html(data);
			}

			$('#SettingsDialog')
				.find('.ep-js')
				.metaparse();

			if (this.ShippingFulfillmentSettingsDialog_1stOpen == true) {
				var InfoBox = $('#LogisticsInfoBoxSlide');
				if (InfoBox.length == 1) {
					InfoBox.hide();
					InfoBox.delay(800)
						.animate({
						opacity: 'toggle',
						height: 'toggle'
					}, 600);
				}

				this.ShippingFulfillmentSettingsDialog_1stOpen = false;
			}

			this.ShippingFulfillmentSettingsDialog.uiDialog('open');
		},

		submit: function (newAction) {
			var ChangeAction = null;
			var oldAction = null;

			if (newAction != null) {
				ChangeAction = $('#SettingsDialog input[name=ChangeAction]');
				oldAction = ChangeAction.val();
				ChangeAction.val(newAction);
			}

			this.lock(true);

			ep.ajax({
				type: "POST",
				url: "?ViewAction=MBO-ShippingFulfillmentSettingsDialog&ObjectID=" + this.ShippingFulfillmentID,
				data: $('#SettingsDialog')
					.serialize()
			})
			.done(function (data) {
				deliverysettingsdialog.reload(data);
        var DialogHasWarnings = $('#ShippingSettingsDialog').find('.DialogHasWarnings').length;
        if (!DialogHasWarnings){
          deliverysettingsdialog.close();
        }
			})
			.fail(function () {
				deliverysettingsdialog.ajaxError();
			})
			.always(function () {
				deliverysettingsdialog.lock(false);
			});

			if (oldAction != null) {
				ChangeAction.val(oldAction);
			}

			var saveButton = this.ShippingFulfillmentSettingsDialog.parent()
					.find(".SaveButton"),
				originalText = saveButton.text(),
				updateSaveButton = function (addClass, removeClass, disable, text) {
					saveButton.addClass(addClass)
						.removeClass(removeClass)
						.attr('disabled', disable)
						.text(text);
				};

			updateSaveButton('Saved', 'SaveButton', true, ep.config.saveButtonChangeMap[originalText]);

			$('#ShippingSettingsDialog')
				.on("change keyup", function () {
					updateSaveButton('SaveButton', 'Saved', false, originalText);
				});
		},

		reload: function (data) {
			this.show(data);
		},

		close: function () {
			this.ShippingFulfillmentSettingsDialog.uiDialog('close');
			DoneShippingFulfillmentSettingsDialog();
		},

		ajaxError: function () {
			ep.alert({
				title: $('#ShippingSettingsDialogLabels > #AlertErrorTitle')
					.html(),
				content: $('#ShippingSettingsDialogLabels > #AlertErrorMessage')
					.html(),
				type: 'warn',
				ok: function (event) {}
			});
		}
	});
});