use strict;
use Test::More tests => 9;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::XML::API::BaseImportHandler;

sub TestNoError {
    my ($XML) =  @_;

    my $Importer = MyImportHandler->new;
    $Importer->parse($XML);
    ok( !ExistsError(), 'no error' );
}

sub TestXMLStackNotEmpty {
    my ($XML) =  @_;

    my $Importer = MyImportHandler->new(
        OnStartDocument => sub {
            my ($this, $doc) = @_;
            $this->pushID('XXX' => '1');
            $this->pushID('XXX' => '2');
        }
    );
    eval { $Importer->parse($XML); };
    ok( ExistsError(), 'ExistsError' );
    my $Error = GetError();
    is( $Error->code, 'XMLStackNotEmpty', 'Error.Code' );
    is( $Error->vars->{'Key'}, 'XXX', 'Error.Vars.Key' );
    is( $Error->vars->{'ValueCount'}, 2, 'Error.Vars.ValueCount' );
    is_deeply( $Error->vars->{'Values'}, [1, 2],  'Error.Vars.Values' );
}

sub TestXMLStackEmpty {
    my ($XML) =  @_;

    my $Importer = MyImportHandler->new(
        OnStartUnknownTag => sub {
            my ($this, $Element) = @_;
            $this->popID('XXX');
        }
    );
    eval { $Importer->parse($XML); };
    ok( ExistsError(), 'ExistsError' );
    my $Error = GetError();
    is( $Error->code, 'XMLStackEmpty', 'Error.Code' );
    is( $Error->vars->{'Key'}, 'XXX', 'Error.Vars.Key' );
}

RunScript( Sub => sub {

    my $XML = <<'END_XML';
<?xml version='1.0' encoding='utf-8'?>
<Company Name='ePages'>
  <Office Country='Germany' City='Jena'>Development Crew</Office>
  <Office Country='Germany' City='Hamburg'>Water Head</Office>
</Company>
END_XML

    TestNoError( $XML );
    TestXMLStackNotEmpty( $XML );
    TestXMLStackEmpty( $XML );
});

package MyImportHandler;
use base 'DE_EPAGES::XML::API::BaseImportHandler';

sub new {
    my $class = shift;
    my %Options = @_;

    my $self = $class->SUPER::new;
    $self->{$_} = $Options{$_} foreach keys %Options;
    return $self;
}

sub parse {
    my $self = shift;
    my ($xml) = @_;

    my $Parser = XML::SAX::ParserFactory->parser( Handler => $self );
    $Parser->parse_string( $xml );
}

sub start_document {
    my $self = shift;
    my ($doc) = @_;

    $self->SUPER::start_document( $doc );

    $self->{OnStartDocument}->($self, $doc) if defined $self->{OnStartDocument};
}

sub startUnknownTag {
    my $self = shift;
    my ($Element) = @_;

    $self->{OnStartUnknownTag}->($self, $Element) if defined $self->{OnStartUnknownTag};
}
